program kde

! Compile: gfortran kde2.f90 -o gokde

  implicit none
  
  character*80 :: input = '1m_radio.dat'
  integer,parameter :: nsmp = 500
  
  integer :: ikern = 1
  ! ikern = 1 : Epanechnikov
  ! ikern = 2 : Quartic
  ! ikern = 3 : Rectangle
  ! ikern = 4 : Gauss
  character*80 :: output = 'pdf_1mradio_epa.dat'
  real :: del0 
  real :: h, sigma,mean
  real,dimension(nsmp) :: axis, pdf
  integer :: ndp, i, j, ios
  real :: dx, x0, hi, mini, maxi, epa, gauss, rect, quart
  real,allocatable :: x(:)

  select case(ikern)
    case(1)
      del0 = 15.0**(0.2) ! Epanechnikov
    case(2)
      del0 = 35.0**(0.2) ! Quartic
    case(3)
      del0 = 4.5**(0.2) ! Rectangle
    case(4)
      del0 = (1.0/(4.0*acos(-1.0)))**(0.1) ! Gauss
  endselect

  print*, ' '
  print*, 'Lese Daten ...'
  
  open(10,file=input,status='old')
  
  ndp = -1
  ios = 0
  
  do while (ios==0)
    read(10,*,iostat=ios) x0
    ndp=ndp+1 
  end do
  
  rewind(10)
  
  print*, ndp, 'Datenpunkte'
  
  allocate(x(ndp))
  
  do i=1,ndp
    read(10,*) x(i)
  end do
  
  close(10)
  print*, 'Fertig.'
  
  print*, ''
  print*, 'Bestimme Wertebereich ...'
  
  mini = x(1)
  maxi = x(1)
  
  do i=2,ndp
    if ( x(i)<mini ) mini = x(i)
    if ( x(i)>maxi ) maxi = x(i)
  end do
  
  mini = mini - (maxi-mini)*0.1d0
  maxi = maxi + (maxi-mini)*0.1d0
  
  print*, 'Minimum:', mini
  print*, 'Maximum:', maxi
  print*, ''
  
  print*, 'Bestimme Koordinatenachse ...'
  
  dx = (maxi - mini)/real(nsmp-1)
  axis(1) = mini
  do i=2,nsmp
    axis(i) = axis(i-1) + dx
  end do
  print*, 'Fertig.'
  
  print*, 'Bestimme optimale Bandbreite ...'
  
  mean = 0.0
  do i=1,ndp
    mean = mean + x(i)
  end do
  mean = mean/real(ndp)
  print*, 'Mittelwert:', mean
  
  sigma = 0.0d0
  do i=1,ndp
    sigma = sigma + (x(i)-mean)**2.0
  end do
  sigma = sigma/real(ndp-1)
  sigma = sqrt(sigma)
  print*, 'Standardabweichung:', sigma
  print*, 'del_0:', del0
  
  h = 1.364*sigma*del0*real(ndp)**(-0.2)
  print*, 'Bandbreite:', h
  
  print*, ''
  print*, 'Schätze pdf ...'  
  
  hi = 1.0/h
  pdf = 0.0
    
  select case(ikern)
    case(1)
      do i=1,nsmp
	x0 = axis(i)
	do j=1,ndp
	  pdf(i) = pdf(i) + epa(hi*(x0-x(j)))
	end do
      end do
    case(2)
      do i=1,nsmp
	x0 = axis(i)
	do j=1,ndp
	  pdf(i) = pdf(i) + quart(hi*(x0-x(j)))
	end do
      end do
    case(3)
      do i=1,nsmp
	x0 = axis(i)
	do j=1,ndp
	  pdf(i) = pdf(i) + rect(hi*(x0-x(j)))
	end do
      end do
    case(4)
      do i=1,nsmp
	x0 = axis(i)
	do j=1,ndp
	  pdf(i) = pdf(i) + gauss(hi*(x0-x(j)))
	end do
      end do
  endselect
  
  pdf = hi*pdf/real(ndp)
  
  print*, 'Fertig.'
  print*, ''
  print*, 'Pruefe Normierung ...'
  
  x0 = 0.0
  do i=1,nsmp
    x0 = x0 + pdf(i)
  end do
  x0 = x0*dx
  print*, 'Norm: ', x0
  
  open(10,file=output,status='replace')
  do i=1,nsmp
    write(10,*) axis(i), pdf(i)
  end do
  close(10)
  
end program

!==========================================================

real function epa(z)

  implicit none

  real z
  real :: c = 0.75

  if (z*z<1.0) then
    epa = c*(1.0 - z*z)
  else
    epa = 0.0
  end if

end function

!=========================================================

real function quart(z)

  implicit none
  
  real z
  real :: c = 15.0/16.0
  
  if (z*z<1.0) then
    quart = c*(1.0 - z*z)**2.0
  else
    quart = 0.0
  end if
  
end function
  
!========================================================
  
real function rect(z)

  implicit none
  
  real z
    
  if (z*z<1.0) then
    rect = 0.5
  else
    rect = 0.0
  end if
  
end function
  
!=======================================================

real function gauss(z)

  implicit none
  
  real z
  real :: pi = acos(-1.0)
  real :: c
  
  c = 1.0/sqrt(2.0*pi)
  
  gauss = c*exp(-0.5*z*z)
  
end function

  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
